"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _TestMessageType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _TestMessageTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_TestMessageType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ERROR: _TestMessageType.ValueType  # 0
    INFO: _TestMessageType.ValueType  # 1

class TestMessageType(_TestMessageType, metaclass=_TestMessageTypeEnumTypeWrapper): ...

ERROR: TestMessageType.ValueType  # 0
INFO: TestMessageType.ValueType  # 1
global___TestMessageType = TestMessageType

class _UploadStatusCode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _UploadStatusCodeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_UploadStatusCode.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Unknown: _UploadStatusCode.ValueType  # 0
    Ok: _UploadStatusCode.ValueType  # 1
    Failed: _UploadStatusCode.ValueType  # 2

class UploadStatusCode(_UploadStatusCode, metaclass=_UploadStatusCodeEnumTypeWrapper): ...

Unknown: UploadStatusCode.ValueType  # 0
Ok: UploadStatusCode.ValueType  # 1
Failed: UploadStatusCode.ValueType  # 2
global___UploadStatusCode = UploadStatusCode

class MLWorkerInfoRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LIST_PACKAGES_FIELD_NUMBER: builtins.int
    list_packages: builtins.bool
    def __init__(
        self,
        *,
        list_packages: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["list_packages", b"list_packages"]) -> None: ...

global___MLWorkerInfoRequest = MLWorkerInfoRequest

class MLWorkerInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class InstalledPackagesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PLATFORM_FIELD_NUMBER: builtins.int
    INTERPRETER_FIELD_NUMBER: builtins.int
    INTERPRETER_VERSION_FIELD_NUMBER: builtins.int
    INSTALLED_PACKAGES_FIELD_NUMBER: builtins.int
    INTERNAL_GRPC_PORT_FIELD_NUMBER: builtins.int
    IS_REMOTE_FIELD_NUMBER: builtins.int
    PID_FIELD_NUMBER: builtins.int
    PROCESS_START_TIME_FIELD_NUMBER: builtins.int
    @property
    def platform(self) -> global___PlatformInfo: ...
    interpreter: builtins.str
    interpreter_version: builtins.str
    @property
    def installed_packages(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    internal_grpc_port: builtins.int
    is_remote: builtins.bool
    pid: builtins.int
    process_start_time: builtins.int
    def __init__(
        self,
        *,
        platform: global___PlatformInfo | None = ...,
        interpreter: builtins.str = ...,
        interpreter_version: builtins.str = ...,
        installed_packages: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        internal_grpc_port: builtins.int = ...,
        is_remote: builtins.bool = ...,
        pid: builtins.int = ...,
        process_start_time: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["platform", b"platform"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["installed_packages", b"installed_packages", "internal_grpc_port", b"internal_grpc_port", "interpreter", b"interpreter", "interpreter_version", b"interpreter_version", "is_remote", b"is_remote", "pid", b"pid", "platform", b"platform", "process_start_time", b"process_start_time"]) -> None: ...

global___MLWorkerInfo = MLWorkerInfo

class PlatformInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MACHINE_FIELD_NUMBER: builtins.int
    NODE_FIELD_NUMBER: builtins.int
    PROCESSOR_FIELD_NUMBER: builtins.int
    RELEASE_FIELD_NUMBER: builtins.int
    SYSTEM_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    machine: builtins.str
    node: builtins.str
    processor: builtins.str
    release: builtins.str
    system: builtins.str
    version: builtins.str
    def __init__(
        self,
        *,
        machine: builtins.str = ...,
        node: builtins.str = ...,
        processor: builtins.str = ...,
        release: builtins.str = ...,
        system: builtins.str = ...,
        version: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["machine", b"machine", "node", b"node", "processor", b"processor", "release", b"release", "system", b"system", "version", b"version"]) -> None: ...

global___PlatformInfo = PlatformInfo

class FileUploadRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    METADATA_FIELD_NUMBER: builtins.int
    CHUNK_FIELD_NUMBER: builtins.int
    @property
    def metadata(self) -> global___FileUploadMetadata: ...
    @property
    def chunk(self) -> global___Chunk: ...
    def __init__(
        self,
        *,
        metadata: global___FileUploadMetadata | None = ...,
        chunk: global___Chunk | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["chunk", b"chunk", "metadata", b"metadata", "request", b"request"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk", b"chunk", "metadata", b"metadata", "request", b"request"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["request", b"request"]) -> typing_extensions.Literal["metadata", "chunk"] | None: ...

global___FileUploadRequest = FileUploadRequest

class FileUploadMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___FileUploadMetadata = FileUploadMetadata

class EchoMsg(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MSG_FIELD_NUMBER: builtins.int
    msg: builtins.str
    def __init__(
        self,
        *,
        msg: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["msg", b"msg"]) -> None: ...

global___EchoMsg = EchoMsg

class ExplainRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    MODEL_FIELD_NUMBER: builtins.int
    DATASET_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    @property
    def dataset(self) -> global___SerializedGiskardDataset: ...
    @property
    def columns(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        model: global___SerializedGiskardModel | None = ...,
        dataset: global___SerializedGiskardDataset | None = ...,
        columns: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["dataset", b"dataset", "model", b"model"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns", "dataset", b"dataset", "model", b"model"]) -> None: ...

global___ExplainRequest = ExplainRequest

class ExplainTextRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class FeatureTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    MODEL_FIELD_NUMBER: builtins.int
    FEATURE_NAME_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    FEATURE_TYPES_FIELD_NUMBER: builtins.int
    N_SAMPLES_FIELD_NUMBER: builtins.int
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    feature_name: builtins.str
    @property
    def columns(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def feature_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    n_samples: builtins.int
    def __init__(
        self,
        *,
        model: global___SerializedGiskardModel | None = ...,
        feature_name: builtins.str = ...,
        columns: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        feature_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        n_samples: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["model", b"model"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns", "feature_name", b"feature_name", "feature_types", b"feature_types", "model", b"model", "n_samples", b"n_samples"]) -> None: ...

global___ExplainTextRequest = ExplainTextRequest

class ExplainResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Explanation(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class PerFeatureEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: builtins.str
            value: builtins.float
            def __init__(
                self,
                *,
                key: builtins.str = ...,
                value: builtins.float = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

        PER_FEATURE_FIELD_NUMBER: builtins.int
        @property
        def per_feature(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.float]: ...
        def __init__(
            self,
            *,
            per_feature: collections.abc.Mapping[builtins.str, builtins.float] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["per_feature", b"per_feature"]) -> None: ...

    class ExplanationsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___ExplainResponse.Explanation: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___ExplainResponse.Explanation | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    EXPLANATIONS_FIELD_NUMBER: builtins.int
    @property
    def explanations(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___ExplainResponse.Explanation]: ...
    def __init__(
        self,
        *,
        explanations: collections.abc.Mapping[builtins.str, global___ExplainResponse.Explanation] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["explanations", b"explanations"]) -> None: ...

global___ExplainResponse = ExplainResponse

class ExplainTextResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ExplanationsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    EXPLANATIONS_FIELD_NUMBER: builtins.int
    @property
    def explanations(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        explanations: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["explanations", b"explanations"]) -> None: ...

global___ExplainTextResponse = ExplainTextResponse

class RunModelForDataFrameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ALL_PREDICTIONS_FIELD_NUMBER: builtins.int
    PREDICTION_FIELD_NUMBER: builtins.int
    PROBABILITIES_FIELD_NUMBER: builtins.int
    RAW_PREDICTION_FIELD_NUMBER: builtins.int
    @property
    def all_predictions(self) -> global___DataFrame: ...
    @property
    def prediction(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def probabilities(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def raw_prediction(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        all_predictions: global___DataFrame | None = ...,
        prediction: collections.abc.Iterable[builtins.str] | None = ...,
        probabilities: collections.abc.Iterable[builtins.float] | None = ...,
        raw_prediction: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["all_predictions", b"all_predictions"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["all_predictions", b"all_predictions", "prediction", b"prediction", "probabilities", b"probabilities", "raw_prediction", b"raw_prediction"]) -> None: ...

global___RunModelForDataFrameResponse = RunModelForDataFrameResponse

class DataRow(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    COLUMNS_FIELD_NUMBER: builtins.int
    @property
    def columns(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        columns: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns"]) -> None: ...

global___DataRow = DataRow

class DataFrame(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROWS_FIELD_NUMBER: builtins.int
    @property
    def rows(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DataRow]: ...
    def __init__(
        self,
        *,
        rows: collections.abc.Iterable[global___DataRow] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["rows", b"rows"]) -> None: ...

global___DataFrame = DataFrame

class RunModelForDataFrameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class FeatureTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class ColumnTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    MODEL_FIELD_NUMBER: builtins.int
    DATAFRAME_FIELD_NUMBER: builtins.int
    TARGET_FIELD_NUMBER: builtins.int
    FEATURE_TYPES_FIELD_NUMBER: builtins.int
    COLUMN_TYPES_FIELD_NUMBER: builtins.int
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    @property
    def dataframe(self) -> global___DataFrame: ...
    target: builtins.str
    @property
    def feature_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def column_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        model: global___SerializedGiskardModel | None = ...,
        dataframe: global___DataFrame | None = ...,
        target: builtins.str = ...,
        feature_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        column_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["dataframe", b"dataframe", "model", b"model"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["column_types", b"column_types", "dataframe", b"dataframe", "feature_types", b"feature_types", "model", b"model", "target", b"target"]) -> None: ...

global___RunModelForDataFrameRequest = RunModelForDataFrameRequest

class RunModelRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_FIELD_NUMBER: builtins.int
    DATASET_FIELD_NUMBER: builtins.int
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    @property
    def dataset(self) -> global___SerializedGiskardDataset: ...
    def __init__(
        self,
        *,
        model: global___SerializedGiskardModel | None = ...,
        dataset: global___SerializedGiskardDataset | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["dataset", b"dataset", "model", b"model"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataset", b"dataset", "model", b"model"]) -> None: ...

global___RunModelRequest = RunModelRequest

class RunModelResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULTS_CSV_FIELD_NUMBER: builtins.int
    CALCULATED_CSV_FIELD_NUMBER: builtins.int
    results_csv: builtins.str
    calculated_csv: builtins.str
    def __init__(
        self,
        *,
        results_csv: builtins.str = ...,
        calculated_csv: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["calculated_csv", b"calculated_csv", "results_csv", b"results_csv"]) -> None: ...

global___RunModelResponse = RunModelResponse

class RunTestRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    MODEL_FIELD_NUMBER: builtins.int
    ACTUAL_DS_FIELD_NUMBER: builtins.int
    REFERENCE_DS_FIELD_NUMBER: builtins.int
    code: builtins.str
    @property
    def model(self) -> global___SerializedGiskardModel: ...
    @property
    def actual_ds(self) -> global___SerializedGiskardDataset: ...
    @property
    def reference_ds(self) -> global___SerializedGiskardDataset: ...
    def __init__(
        self,
        *,
        code: builtins.str = ...,
        model: global___SerializedGiskardModel | None = ...,
        actual_ds: global___SerializedGiskardDataset | None = ...,
        reference_ds: global___SerializedGiskardDataset | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["actual_ds", b"actual_ds", "model", b"model", "reference_ds", b"reference_ds"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["actual_ds", b"actual_ds", "code", b"code", "model", b"model", "reference_ds", b"reference_ds"]) -> None: ...

global___RunTestRequest = RunTestRequest

class SerializedGiskardDataset(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class FeatureTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class ColumnTypesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    TARGET_FIELD_NUMBER: builtins.int
    FEATURE_TYPES_FIELD_NUMBER: builtins.int
    COLUMN_TYPES_FIELD_NUMBER: builtins.int
    SERIALIZED_DF_FIELD_NUMBER: builtins.int
    target: builtins.str
    @property
    def feature_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def column_types(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    serialized_df: builtins.bytes
    def __init__(
        self,
        *,
        target: builtins.str = ...,
        feature_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        column_types: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        serialized_df: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["column_types", b"column_types", "feature_types", b"feature_types", "serialized_df", b"serialized_df", "target", b"target"]) -> None: ...

global___SerializedGiskardDataset = SerializedGiskardDataset

class RunTestResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___RunTestResponse = RunTestResponse

class Partial_unexpected_counts(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    COUNT_FIELD_NUMBER: builtins.int
    @property
    def value(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    count: builtins.int
    def __init__(
        self,
        *,
        value: collections.abc.Iterable[builtins.int] | None = ...,
        count: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count", b"count", "value", b"value"]) -> None: ...

global___Partial_unexpected_counts = Partial_unexpected_counts

class NamedSingleTestResult(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    RESULT_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def result(self) -> global___SingleTestResult: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        result: global___SingleTestResult | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["result", b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "result", b"result"]) -> None: ...

global___NamedSingleTestResult = NamedSingleTestResult

class TestMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    TEXT_FIELD_NUMBER: builtins.int
    type: global___TestMessageType.ValueType
    text: builtins.str
    def __init__(
        self,
        *,
        type: global___TestMessageType.ValueType = ...,
        text: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["text", b"text", "type", b"type"]) -> None: ...

global___TestMessage = TestMessage

class SingleTestResult(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class PropsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PASSED_FIELD_NUMBER: builtins.int
    MESSAGES_FIELD_NUMBER: builtins.int
    PROPS_FIELD_NUMBER: builtins.int
    METRIC_FIELD_NUMBER: builtins.int
    MISSING_COUNT_FIELD_NUMBER: builtins.int
    MISSING_PERCENT_FIELD_NUMBER: builtins.int
    UNEXPECTED_COUNT_FIELD_NUMBER: builtins.int
    UNEXPECTED_PERCENT_FIELD_NUMBER: builtins.int
    UNEXPECTED_PERCENT_TOTAL_FIELD_NUMBER: builtins.int
    UNEXPECTED_PERCENT_NONMISSING_FIELD_NUMBER: builtins.int
    PARTIAL_UNEXPECTED_INDEX_LIST_FIELD_NUMBER: builtins.int
    PARTIAL_UNEXPECTED_COUNTS_FIELD_NUMBER: builtins.int
    UNEXPECTED_INDEX_LIST_FIELD_NUMBER: builtins.int
    OUTPUT_DF_FIELD_NUMBER: builtins.int
    NUMBER_OF_PERTURBED_ROWS_FIELD_NUMBER: builtins.int
    ACTUAL_SLICES_SIZE_FIELD_NUMBER: builtins.int
    REFERENCE_SLICES_SIZE_FIELD_NUMBER: builtins.int
    passed: builtins.bool
    @property
    def messages(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TestMessage]: ...
    @property
    def props(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    metric: builtins.float
    @property
    def missing_count(self) -> google.protobuf.wrappers_pb2.Int32Value: ...
    @property
    def missing_percent(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def unexpected_count(self) -> google.protobuf.wrappers_pb2.Int32Value: ...
    @property
    def unexpected_percent(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def unexpected_percent_total(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def unexpected_percent_nonmissing(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def partial_unexpected_index_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def partial_unexpected_counts(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Partial_unexpected_counts]: ...
    @property
    def unexpected_index_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    output_df: builtins.bytes
    number_of_perturbed_rows: builtins.int
    @property
    def actual_slices_size(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def reference_slices_size(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        passed: builtins.bool = ...,
        messages: collections.abc.Iterable[global___TestMessage] | None = ...,
        props: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        metric: builtins.float = ...,
        missing_count: google.protobuf.wrappers_pb2.Int32Value | None = ...,
        missing_percent: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        unexpected_count: google.protobuf.wrappers_pb2.Int32Value | None = ...,
        unexpected_percent: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        unexpected_percent_total: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        unexpected_percent_nonmissing: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        partial_unexpected_index_list: collections.abc.Iterable[builtins.int] | None = ...,
        partial_unexpected_counts: collections.abc.Iterable[global___Partial_unexpected_counts] | None = ...,
        unexpected_index_list: collections.abc.Iterable[builtins.int] | None = ...,
        output_df: builtins.bytes = ...,
        number_of_perturbed_rows: builtins.int = ...,
        actual_slices_size: collections.abc.Iterable[builtins.int] | None = ...,
        reference_slices_size: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["missing_count", b"missing_count", "missing_percent", b"missing_percent", "unexpected_count", b"unexpected_count", "unexpected_percent", b"unexpected_percent", "unexpected_percent_nonmissing", b"unexpected_percent_nonmissing", "unexpected_percent_total", b"unexpected_percent_total"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["actual_slices_size", b"actual_slices_size", "messages", b"messages", "metric", b"metric", "missing_count", b"missing_count", "missing_percent", b"missing_percent", "number_of_perturbed_rows", b"number_of_perturbed_rows", "output_df", b"output_df", "partial_unexpected_counts", b"partial_unexpected_counts", "partial_unexpected_index_list", b"partial_unexpected_index_list", "passed", b"passed", "props", b"props", "reference_slices_size", b"reference_slices_size", "unexpected_count", b"unexpected_count", "unexpected_index_list", b"unexpected_index_list", "unexpected_percent", b"unexpected_percent", "unexpected_percent_nonmissing", b"unexpected_percent_nonmissing", "unexpected_percent_total", b"unexpected_percent_total"]) -> None: ...

global___SingleTestResult = SingleTestResult

class TestResultMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULTS_FIELD_NUMBER: builtins.int
    @property
    def results(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NamedSingleTestResult]: ...
    def __init__(
        self,
        *,
        results: collections.abc.Iterable[global___NamedSingleTestResult] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["results", b"results"]) -> None: ...

global___TestResultMessage = TestResultMessage

class SerializedGiskardModel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERIALIZED_PREDICTION_FUNCTION_FIELD_NUMBER: builtins.int
    MODEL_TYPE_FIELD_NUMBER: builtins.int
    THRESHOLD_FIELD_NUMBER: builtins.int
    FEATURE_NAMES_FIELD_NUMBER: builtins.int
    CLASSIFICATION_LABELS_FIELD_NUMBER: builtins.int
    serialized_prediction_function: builtins.bytes
    model_type: builtins.str
    @property
    def threshold(self) -> google.protobuf.wrappers_pb2.DoubleValue: ...
    @property
    def feature_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def classification_labels(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        serialized_prediction_function: builtins.bytes = ...,
        model_type: builtins.str = ...,
        threshold: google.protobuf.wrappers_pb2.DoubleValue | None = ...,
        feature_names: collections.abc.Iterable[builtins.str] | None = ...,
        classification_labels: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["threshold", b"threshold"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["classification_labels", b"classification_labels", "feature_names", b"feature_names", "model_type", b"model_type", "serialized_prediction_function", b"serialized_prediction_function", "threshold", b"threshold"]) -> None: ...

global___SerializedGiskardModel = SerializedGiskardModel

class Chunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    content: builtins.bytes
    def __init__(
        self,
        *,
        content: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["content", b"content"]) -> None: ...

global___Chunk = Chunk

class UploadStatus(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    code: global___UploadStatusCode.ValueType
    def __init__(
        self,
        *,
        code: global___UploadStatusCode.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code", b"code"]) -> None: ...

global___UploadStatus = UploadStatus
